/**************************************************/
/***						***/
/***	coach.c					***/
/***						***/
/**************************************************/

/* gcc -ggdb -O -o robo1 main1.c coach.c util.c shmem.c -lm */

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>

#include	"main.h"
#include	"util.h"
#include	"function.h"
#include	"flag.h"
#include	"action.h"

#ifdef		MAIN
#define		EXTERN
#else
#define         EXTERN		extern
#endif

EXTERN	Obj	ball, mate[MAXPLAYER], enemy[MAXPLAYER];
EXTERN	int	nmate, nenemy;
EXTERN	CmdList gCmdList[MAXPLAYER];
extern	Obj	gMyGoal, gEnemyGoal, gFutureBall;

FLOAT		gPredictedBallTime;

void	HeadCoachInit(void)
{
#ifndef SIMULATE
    gPredictedBallTime = PREDICTTIMEBALL;
#endif
    InitVar(&ball, mate, enemy, nmate, nenemy);
    MakeStaticPotentialField();
/*  StopRobot();*/
}

void	HeadCoach(int mode)
{
    int		i;
    static int	command[MAXPLAYER][5], time = 0;

#ifdef	USENOP
    static int	count = 0;

    if (DistanceFromXY(ball.x, ball.y, ball.px, ball.py) < 1.0 && count < 4) {
        for (i = 0; i < nmate; i++) Nop(&(mate[i]));
	SendCmdList();
#ifdef	PRINTLOG
    PrintTeamLog(&ball, mate, enemy, gCmdList, time++);
#endif
        count++;
        return;
    }
    else {
        count = 0;
    }
#endif

    switch (mode) {
    case ST_PKKICK:	StrategyPKKick(command); break;
    case ST_PKBLOCK:	StrategyPKBlock(command); break;
    case ST_PKBLOCKLEFT:  StrategyPKBlockLeft(command); break;
    case ST_PKBLOCKRIGHT: StrategyPKBlockRight(command); break;
    case ST_1:		Strategy1(command);	break;
    case ST_31:		Strategy31(command);	break;
    case ST_32:		Strategy32(command);	break;
    case ST_5TEST:	Strategy5test(command);	break;
    case ST_51:		Strategy51(command);	break;
    case ST_52:		Strategy52(command);	break;
    case ST_52A:	Strategy52a(command);	break;
    case ST_PTEST:	StrategyPtest(command);	break;
    case ST_P51:	StrategyP51(command);	break;
    case ST_P52:	StrategyP52(command);	break;
    case ST_NTEST:	StrategyNtest(command);	break;
    case ST_DEFAULT:
    default:		Strategy52(command);    break;
    }
    Action(mate, command[0]);
    Action(mate, command[1]);
    Action(mate, command[2]);
    Action(mate, command[3]);
    Action(mate, command[4]);	
    SendCmdList();
#ifdef	PRINTLOG
    PrintTeamLog(&ball, mate, enemy, gCmdList, time++);
#endif
}

void	HeadCoachOld(void)
{
    int		code;
    static int	count = 0;
    static int	command[MAXPLAYER][5];

    Strategy52(command);

    code = count % 2;
    count++;

    switch (code) {
    case 0:	
	Strategy52(command);		break;
    case 1:
    	Action(mate, command[0]);
    	Action(mate, command[1]);
    	Action(mate, command[2]);
    	Action(mate, command[3]);
    	Action(mate, command[4]);	
	SendCmdList();                  break;
    default:	Strategy52(command); 	break;
    }	
}

void	HeadCoachTest(void)
{
    int		code;
    static int	count = 0;

    code = count % 6;
    count++;

    switch (code) {
    case 0:
	NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y),
			 &(gFutureBall. dir));
	break;
    case 1:
	if (1 <= nmate) PressBall(&gFutureBall, &(mate[0])); 
	break;
    default:
	break;
    }	
}

/*
 * StrategyPKKick:
 * ball ˰ֶᤤܥåȤå롣
 */

void	StrategyPKKick(int command[][5])
{

    static int	id[MAXPLAYER] = {0, 1, 2, 3, 4};
    FLOAT	d[MAXPLAYER];

    OrderBallPlayer(mate, nmate, id, d);

    command[0][0] = id[0];
    command[0][1] = PKKICK;
    command[1][0] = id[1];
    command[1][1] = STAY;
    command[2][0] = id[2];
    command[2][1] = STAY;
    command[3][0] = id[3];
    command[3][1] = STAY;
    command[4][0] = id[4];
    command[4][1] = STAY;
}

/*
 * StrategyPKBlock:
 * PK  GK ư¾ΥܥåȤϻߤޤäƤ롣GK  0 
 */

void	StrategyPKBlock(int command[][5])
{
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y),
                     &(gFutureBall. dir));

    command[0][0] = 0;
    command[0][1] = PKBLOCK;
    command[1][0] = 1;
    command[1][1] = STAY;
    command[2][0] = 2;
    command[2][1] = STAY;
    command[3][0] = 3;
    command[3][1] = STAY;
    command[4][0] = 4;
    command[4][1] = STAY;
}

/*
 * StrategyPKBlockLeft:
 * PK  GK ư(Left)¾ΥܥåȤϻߤޤäƤ롣GK  0 
 */

void	StrategyPKBlockLeft(int command[][5])
{
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y),
                     &(gFutureBall. dir));

    command[0][0] = 0;
    command[0][1] = PKBLOCKLEFT;
    command[1][0] = 1;
    command[1][1] = STAY;
    command[2][0] = 2;
    command[2][1] = STAY;
    command[3][0] = 3;
    command[3][1] = STAY;
    command[4][0] = 4;
    command[4][1] = STAY;
}

/*
 * StrategyPKBlockRight:
 * PK  GK ư(Right)¾ΥܥåȤϻߤޤäƤ롣GK  0 
 */

void	StrategyPKBlockRight(int command[][5])
{
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y),
                     &(gFutureBall. dir));

    command[0][0] = 0;
    command[0][1] = PKBLOCKRIGHT;
    command[1][0] = 1;
    command[1][1] = STAY;
    command[2][0] = 2;
    command[2][1] = STAY;
    command[3][0] = 3;
    command[3][1] = STAY;
    command[4][0] = 4;
    command[4][1] = STAY;
}

/*
 * Strategy1:
 * ؿǻꤷܥå 1 ץ쥹
 */

void	Strategy1(int command[][5])
{
    int		i, target = 0;

    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y),
                     &(gFutureBall. dir));
    for (i = 0; i < MAXPLAYER; i++) {
	command[i][0] = i;
	command[i][1] = STAY;
    }
    command[target][1] = SelectPress(BallArea());
}

/*
 * Strategy52a:
 * 5 ǥܡ˶ᤤ 2 椬˥ץ쥹롣
 * Ĥ 3 Ŭڤʰ֤˰ư롣
 */

void	Strategy52a(int command[][5])
{
    static int	id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int		ballarea, side, status, clist[MAXPLAYER];
    FLOAT	d[MAXPLAYER];

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;

    AllObjArea(area);

    status = FieldStatus(ballarea, area);

    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(mate, nmate, id, d);
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(mate, nmate, id, d);
#endif

    RestPlayer1(mate, nmate - 2, &(id[2]), &(d[2]), side, status, &(clist[2]));

    command[0][0] = id[0];
    command[0][1] = SelectPress(ballarea);
    command[1][0] = id[1];
    command[1][1] = SelectPress(ballarea);
    command[2][0] = id[2];
    command[2][1] = clist[2];
    command[3][0] = id[3];
    command[3][1] = clist[3];
    command[4][0] = id[4];
    command[4][1] = clist[4];
}

/*
 * Strategy52:
 * ѡʳ 4 ǥܡ˶ᤤ 2 椬˥ץ쥹롣
 * Ĥ 2 Ŭڤʰ֤˰ư롣
 */

void	Strategy52(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int		ballarea, side, status, clist[MAXPLAYER];
    FLOAT	d[MAXPLAYER];

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;

    AllObjArea(area);

    status = FieldStatus(ballarea, area);

    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    RestPlayer2(mate, 2, &(id[2]), &(d[2]), side, status, &(clist[2]));

    if (ballarea <= 6) {
	command[0][0] = 0;
	command[0][1] = GKBLOCK;
	command[1][0] = id[0];
	command[1][1] = SelectPress(ballarea);
	command[2][0] = id[1];
	command[2][1] = SelectPress(ballarea);
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
    else {
	command[0][0] = id[0];
	command[0][1] = SelectPress(ballarea);
	command[1][0] = id[1];
	command[1][1] = SelectPress(ballarea);
	command[2][0] = 0;
	command[2][1] = KEEPERPOSITION;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
}

/*
 * Strategy51:
 * ѡʳ 4 ǥܡ˶ᤤ 2 椬˥ץ쥹롣
 * Τ 1 ϾԵ롣
 * Ĥ 2 Ŭڤʰ֤˰ư롣
 */

void	Strategy51(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         i, bk, area[20], tmp;
    int		ballarea, side, status, clist[MAXPLAYER];
    FLOAT       x, y;
    FLOAT	d[MAXPLAYER], dist[MAXPLAYER];

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;

    AllObjArea(area);

    status = FieldStatus(ballarea, area);

    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    if (CX < ball. x) {
        for (i = 0; i < nmate; i++) dist[i] = 300;
	for (i = 0; i < nmate - 1; i++) dist[id[i]] = d[i];
        DiredPoint(ball. x, ball. y, 
		   DirFromXY(gEnemyGoal. x, gEnemyGoal. y, ball. x, ball. y), 
		   10.0, &x, &y);
	bk = WhoisBestKicker(mate, dist, x, y);
	if (-1 != bk) {
	    for (i = 0; i < nmate; i++) {
	        if (bk == id[i]) {tmp = id[0]; id[0] = id[i]; id[i] = tmp;}
	    }
	}  
    }
    RestPlayer2(mate, 2, &(id[2]), &(d[2]), side, status, &(clist[2]));

    if (ballarea <= 6) {
	command[0][0] = 0;
	command[0][1] = GKBLOCK;
	command[1][0] = id[0];
	command[1][1] = SelectPress(ballarea);
	command[2][0] = id[1];
	command[2][1] = PRESSBALL2;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
    else {
	command[0][0] = id[0];
	command[0][1] = SelectPress(ballarea);
	command[1][0] = id[1];
	command[1][1] = PRESSBALL2;
	command[2][0] = 0;
	command[2][1] = KEEPERPOSITION;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
}

/*
 * Strategy5test
 * ѡʳ 4 ǥܡ˶ᤤ 1 椬˥ץ쥹롣
 * ѡưĤ 3  STAYƥѥ롼
 */

void	Strategy5test(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int		ballarea, side, status;
    FLOAT	d[MAXPLAYER];

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;

    AllObjArea(area);

    status = FieldStatus(ballarea, area);

    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    if (ballarea <= 6) {
	command[0][0] = 0;
	command[0][1] = GKBLOCK;
	command[1][0] = id[0];
	command[1][1] = SelectPress(ballarea);
	command[2][0] = id[1];
	command[2][1] = STAY;
	command[3][0] = id[2];
	command[3][1] = STAY;
	command[4][0] = id[3];
	command[4][1] = STAY;
    }
    else {
	command[0][0] = id[0];
	command[0][1] = SelectPress(ballarea);
	command[1][0] = 0;
	command[1][1] = KEEPERPOSITION;
	command[2][0] = id[1];
	command[2][1] = STAY;
	command[3][0] = id[2];
	command[3][1] = STAY;
	command[4][0] = id[3];
	command[4][1] = STAY;
    }
}

/*
 * StrategyNtest:
 * nmate ǥܡ˶ᤤ 1 椬˥ץ쥹롣
 * Ĥ nmate - 1  STAY
 */

void	StrategyNtest(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int		ballarea, side, status;
    FLOAT	d[MAXPLAYER];

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;
    AllObjArea(area);
    status = FieldStatus(ballarea, area);
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    command[0][0] = id[0];
    command[0][1] = SelectPress(ballarea);
    command[1][0] = id[1];
    command[1][1] = STAY;
    command[2][0] = id[2];
    command[2][1] = STAY;
    command[3][0] = id[3];
    command[3][1] = STAY;
    command[4][0] = id[4];
    command[4][1] = STAY;
}

/*
 * StrategyPtest:
 * Potential Field ˴Ťƹư롣
 * ܡ˶ᤤ 1 椬ݥƥ󥷥ץ쥹ĤϻߤޤäƤ
 */

void	StrategyPtest(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int		side, ballarea;
    FLOAT	d[MAXPLAYER];

    if (6 < BallArea()) side = ENEMYSIDE;
    else	      	side = MYSIDE;

    GetPotentialField(mate, enemy, nmate, nenemy);

    ballarea = BallArea();
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    command[0][0] = id[0];
    command[0][1] = POTENTIALPRESS;
    command[1][0] = id[1];
    command[1][1] = STAY;
    command[2][0] = id[2];
    command[2][1] = STAY;
    command[3][0] = id[3];
    command[3][1] = STAY;
    command[4][0] = id[4];
    command[4][1] = STAY;
}

/*
 * StrategyP52:
 * ѡʳ 4 ǥܡ˶ᤤ 2 椬˥ݥƥ󥷥ץ쥹롣
 * Ĥ 2 Ŭڤʰ֤˰ư롣
 */

void	StrategyP52(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int		side, status, ballarea, clist[MAXPLAYER];
    FLOAT	d[MAXPLAYER];

    GetPotentialField(mate, enemy, nmate, nenemy);

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;
    AllObjArea(area);
    status = FieldStatus(ballarea, area);
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    RestPlayer2(mate, 2, &(id[2]), &(d[2]), side, status, &(clist[2]));

    if (ballarea <= 6) {
	command[0][0] = 0;
	command[0][1] = GKBLOCK;
	command[1][0] = id[0];
	command[1][1] = POTENTIALPRESS;
	command[2][0] = id[1];
	command[2][1] = POTENTIALPRESS;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
    else {
	command[0][0] = id[0];
	command[0][1] = POTENTIALPRESS;
	command[1][0] = id[1];
	command[1][1] = POTENTIALPRESS;
	command[2][0] = 0;
	command[2][1] = KEEPERPOSITION;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
}

/*
 * StrategyP51:
 * ѡʳ 4 ǥܡ˶ᤤ 2 椬˥ݥƥ󥷥ץ쥹롣
 * Τ 1 ϾԵ롣
 * Ĥ 2 Ŭڤʰ֤˰ư롣
 */

void	StrategyP51(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int		side, status, ballarea, clist[MAXPLAYER];
    FLOAT	d[MAXPLAYER];

    GetPotentialField(mate, enemy, nmate, nenemy);

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else	      side = MYSIDE;
    AllObjArea(area);
    status = FieldStatus(ballarea, area);
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(&(mate[1]), nmate - 1, id, d); /* except GK */
#endif

    RestPlayer2(mate, 2, &(id[2]), &(d[2]), side, status, &(clist[2]));

    if (ballarea <= 6) {
	command[0][0] = 0;
	command[0][1] = GKBLOCK;
	command[1][0] = id[0];
	command[1][1] = POTENTIALPRESS;
	command[2][0] = id[1];
	command[2][1] = PRESSBALL2;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
    else {
	command[0][0] = id[0];
	command[0][1] = POTENTIALPRESS;
	command[1][0] = id[1];
	command[1][1] = PRESSBALL2;
	command[2][0] = 0;
	command[2][1] = KEEPERPOSITION;
	command[3][0] = id[2];
	command[3][1] = clist[2];
	command[4][0] = id[3];
	command[4][1] = clist[3];
    }
}

/*
 * Strategy31:
 * 3 ǥܡ˶ᤤ 1 椬˥ץ쥹롣
 * Ǥ 1 椬 FW Ԥ1  DF ˤʤ롣
 * Ǥ 1 椬 DF Ԥ1  GK ˤʤ롣
 */

void	Strategy31(int command[][5])
{
    static int  id[MAXPLAYER] = {0, 1, 2, 3, 4};
    int         area[20];
    int         ballarea, side, status, clist[MAXPLAYER];
    FLOAT       d[MAXPLAYER];

    ballarea = BallArea();
    if (6 < ballarea) side = ENEMYSIDE;
    else              side = MYSIDE;
    AllObjArea(area);
    status = FieldStatus(ballarea, area);
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

#ifdef  PREDICTFUTURE
    OrderFutureBallPlayer(mate, 3, id, d); /* except GK */
#endif
#ifdef  CURRENTONLY
    OrderBallPlayer(mate, 3, id, d); /* except GK */
#endif

    RestPlayer2(mate, 2, &(id[1]), &(d[1]), side, status, &(clist[1]));

    if (ballarea <= 6) {
	command[0][0] = id[0];
	command[0][1] = SelectPress(ballarea);
	command[1][0] = id[1];
	command[1][1] = GKBLOCK;
	command[2][0] = id[2];
	command[2][1] = DFSTANDARD;
	command[3][0] = 3;
	command[3][1] = STAY;
	command[4][0] = 4;
	command[4][1] = STAY;
    }
    else {
	command[0][0] = id[0];
	command[0][1] = SelectPress(ballarea);
	command[1][0] = id[1];
	command[1][1] = FREESPACE;
	command[2][0] = id[2];
	command[2][1] = DFSTANDARD;
	command[3][0] = 3;
	command[3][1] = STAY;
	command[4][0] = 4;
	command[4][1] = STAY;
    }
} 

/*
 * Strategy32:
 * 3 Ѥάǡ1  GK ꡢĤ 2 Ͼ˥ץ쥹롣
 */

void	Strategy32(int command[][5])
{
    int         ballarea;

    ballarea = BallArea();
    NextBallPosition(&ball, gPredictedBallTime, &(gFutureBall. x), &(gFutureBall. y), 
		     &(gFutureBall. dir));

    if (ballarea <= 6) {
	command[0][0] = 0;
	command[0][1] = GKBLOCK;
	command[1][0] = 1;
	command[1][1] = SelectPress(ballarea);
	command[2][0] = 2;
	command[2][1] = SelectPress(ballarea);
	command[3][0] = 3;
	command[3][1] = STAY;
	command[4][0] = 4;
	command[4][1] = STAY;
    }
    else {
	command[0][0] = 1;
	command[0][1] = SelectPress(ballarea);
	command[1][0] = 2;
	command[1][1] = SelectPress(ballarea);
	command[2][0] = 0;
	command[2][1] = KEEPERPOSITION;
	command[3][0] = 3;
	command[3][1] = STAY;
	command[4][0] = 4;
	command[4][1] = STAY;
    }
} 

/*
 * MoveTest:
 * 1 ưƥ
 */

void	MoveTest(int mode)
{
    static int	start = 0;
    static int	target = 0;
    static FLOAT x[4], y[4];

    if (0 == start) {
        start = 1;
        x[0] = mate[0]. x;	y[0] = mate[0]. y;
        x[1] = mate[0]. x + 50;	y[1] = mate[0]. y;
        x[2] = mate[0]. x + 50;	y[2] = mate[0]. y + 50;
        x[3] = mate[0]. x;	y[3] = mate[0]. y + 50;
    }
    if (DistanceFromXY(mate[0]. x, mate[0]. y, x[target], y[target]) < 5) {
        target++;
	if (4 == target) target = 0;
    }
    GoXY(&(mate[0]), x[target], y[target]);

    SendCmdList();
} 

void	RestPlayer1(Obj *player, int num, int id[], FLOAT d[], int side, int status, int command[])
{
    int		tmp;

    if (MYSIDE == side) {
	if (player[id[1]]. x < player[id[0]]. x) {
	    tmp = id[0]; id[0] = id[1]; id[1] = tmp;
	}
	if (player[id[2]]. x < player[id[1]]. x) {
	    tmp = id[1]; id[1] = id[2]; id[2] = tmp;
	}
	command[0] = KEEPERPOSITION;
	command[1] = DFPOSITION;
	command[2] = WAITFW;
    }
    else { /* (ENEMYSIDE == side) */
	if (player[id[0]]. x < player[id[1]]. x) {
	    tmp = id[0]; id[0] = id[1]; id[1] = tmp;
	}
	if (player[id[1]]. x < player[id[2]]. x) {
	    tmp = id[1]; id[1] = id[2]; id[2] = tmp;
	}
	command[0] = FREESPACE;
	command[1] = DFSTANDARD;
	command[2] = KEEPERPOSITION;
    }
}

void	RestPlayer2(Obj *player, int num, int id[], FLOAT d[], int side, int status, int command[])
{
    int		tmp;

    if (MYSIDE == side) {
	if (player[id[1]]. x < player[id[0]]. x) {
	    tmp = id[0]; id[0] = id[1]; id[1] = tmp;
	}
	command[0] = DFPOSITION;
	command[1] = WAITFW;
    }
    else { /* (ENEMYSIDE == side) */
	if (player[id[0]]. x < player[id[1]]. x) {
	    tmp = id[0]; id[0] = id[1]; id[1] = tmp;
	}
	command[0] = WAITFW; // FREESPACE;
	command[1] = DFSTANDARD;
    }
}

int	SelectPress(int ballarea)
{
    if (7 == ballarea || 10 == ballarea)
        return(SHOOT);
    if (1 == ballarea || 3 == ballarea || 4 == ballarea) 
        return(CLEAR);
    if (11 == ballarea || 12 == ballarea)
        return(CENTERING);
    if (5 == ballarea || 6 == ballarea || 8 == ballarea || 9 == ballarea)
	return(SIDEATTACK);

    return(PRESSBALL);

//  return(POTENTIALPRESS);
}
