#include <stdio.h>
#include "main.h"
#include "serial.h"

#include "prottype.h"

void tread(int fd, char *buff, int num){
  int i,j,nread;
  char buff1[16];

  i= 0;
  while(i<num) {
    nread= read(fd,buff1,num);
    /* printf("DBG:nread=%d\n",nread);*/
    if(nread!=0) {
      for(j=0;j<nread;j++) buff[i++]=buff1[j];
    } else {
      usleep(100); /*sleep 1000micro sec*/
    }
  }
}


int modem_show_reg(int fd, int reg_no){
  char buff[16];
  int i,error;

  error= 0;
  sprintf(buff,"@REG%02d\15\12",reg_no);
  write(fd,buff,8);
  tread(fd,buff,5);
  fprintf(stderr,"REG%02d=",reg_no);
  for(i=0;i<5;i++) fprintf(stderr,"%c",buff[i]&0xFF);
  fprintf(stderr,"\n");
  return(error);
}


int modem_set_reg(int fd, int reg_no, int val){
  char buff[16];
  int i,error;

  error= 0;
  sprintf(buff,"@REG%02d:%02xH\15\12",reg_no,val);
  write(fd,buff,12);
  tread(fd,buff,5);
  if(buff[0]=='P') {
    if(buff[1]!='0') error=9;
  } 
  else {
    switch(buff[1]) {
    case '0':
      error=1;
      break;
    case '6':
      error=2;
      break;
    default:
      error=9;
    }
  }
  switch(error) {
  case 1:
    fprintf(stderr,"REG: command error\n");
    break;
  case 2:
    fprintf(stderr,"REG: memory register write error\n");
    break;
  case 9:
    fprintf(stderr,"REG: illeagal return code.");
    for(i=0;i<4;i++) printf("%3x",buff[i]&0xFF);
    fprintf(stderr,"\n");
    break;
  }
  return(error);
}


int modem_bcl(int fd){
  char buff[16];
  int i,error;

  error= 0;
  /*Clear buffer*/
  write(fd,"@BCL\015\012",6);
  tread(fd,buff,4);
  if(buff[0]=='P') {
    if(buff[1]!='0') error=9;
  } else {
    if(buff[1]=='0') error=1; else error=9;
  }
  switch(error) {
  case 1:
    fprintf(stderr,"BCL: command error\n");
    break;
  case 9:
    fprintf(stderr,"BCL: illeagal return code\n");
    for(i=0;i<4;i++) printf("%3x",buff[i]&0xFF);
    fprintf(stderr,"\n");
    break;
  }
  return(error);
}


int modem_rst(int fd){
  char buff[16];
  int i,error;

  error= 0;
  /*Clear buffer*/
  write(fd,"@RST\015\012",6);
  tread(fd,buff,4);
  if(buff[0]=='P') {
    if(buff[1]!='0') error=9;
  } else {
    if(buff[1]=='0') error=1; else error=9;
  }
  switch(error) {
  case 1:
    fprintf(stderr,"RST: command error\n");
    break;
  case 9:
    fprintf(stderr,"RST: illeagal return code\n");
    for(i=0;i<4;i++) printf("%3x",buff[i]&0xFF);
    fprintf(stderr,"\n");
    break;
  }
  return(error);
}


int modem_dcn(int fd){
  char buff[16];
  int i,cnt,error;

  /*disconnect robo*/
  cnt= 0; error= 0;
LBL00:
  write(fd,"@DCN\015\012",6);
LBL1:
  tread(fd,buff,4);
  switch(buff[0]) {
  case 'P':
    if(buff[1]=='1') goto LBL1;
    if(buff[1]!='0') error=9;
    break;
  case 'N':
    switch( buff[1] ) {
    case '0':
      error= 1;
      break;
    case '1':
      cnt++; 
      if(cnt<4) goto LBL00; else error= 2;
      break;
    case '4': /*already disconnected the line*/
      break;
    default:
      error= 9;
    }
    break;
  default:
    error= 9;
  }
  switch(error) {
  case 1:
    fprintf(stderr,"DCN: command error\n");
    break;
  case 2:
    fprintf(stderr,"DCN: repeated 3 times but cannot disconnect\n");
    break;
  case 9:
    fprintf(stderr,"DCN: illeagal return code\n");
    for(i=0;i<4;i++) printf("%3x",buff[i]&0xFF);
    fprintf(stderr,"\n");
    break;
  }
  return(error);
}


int modem_con(int fd,int robo_NO){
  char buff[16];
  int i,error;

  error= 0;
  /*connect robo1*/
  sprintf(buff,"@CON%03d\015\012",robo_NO);
  write(fd,buff,9);
LBL1:
  tread(fd,buff,4);
  switch(buff[0]) {
  case 'P':
    if(buff[1]=='1') goto LBL1;
    if(buff[1]!='0') error=9;
    break;
  case 'N':
    switch( buff[1] ) {
    case '0':
      error= 1;
      break;
    case '1':
      error= 2;
      break;
    case '4':
      error= 3;
      break;
    default:
      error= 9;
    }
    break;
  default:
    error= 9;
  }

  switch(error) {
  case 1:
    fprintf(stderr,"CON: command error\n");
    break;
  case 2: 
    fprintf(stderr,"CON: robot-side-modem returned no response\n");
    break;
  case 3: 
    fprintf(stderr,"CON: line is busy\n");
    break;
  case 9:
    fprintf(stderr,"CON: illeagal return code\n");
    for(i=0;i<4;i++) fprintf(stderr,"%3x",buff[i]&0xFF);
    fprintf(stderr,"\n");
    break;
  }
  return(error);
}









