/**************************************************/
/***						***/
/***	potential.c				***/
/***						***/
/**************************************************/

/* gcc -ggdb -O -o robo1 main1.c coach.c potential.c util.c shmem.c -lm */

#include	<stdio.h>
#include	<stdlib.h>
#include        <string.h>
#include	<math.h>

#include	"main.h"
#include	"util.h"
#include	"function.h"
#include	"flag.h"

#ifdef		MAIN
#define		EXTERN
#else
#define         EXTERN		extern
#endif

#define		PLUS		1
#define		MINUS		-1
#define		GOALPTL		5	/* potential of goal */
#define		PLAYERPTL	1	/* potential of player */
#define		PAREA		10	/* radius of potential area [cm] */
#define		MAXPOTENTIAL	1

FLOAT		gDummy1[PX][PY];	/* protector 4 memory leak */
FLOAT		gStaticField[PX][PY];
FLOAT		gActualField[PX][PY];
FLOAT           gDummy2[PX][PY];	/* protector 4 memory leak */

void	GetPotentialField(Obj *mate, Obj *enemy, int nmate, int nenemy)
{
//  static int	save = 0;

    CopyPotentialField((void *)gStaticField, (void *)gActualField, PX, PY);

    CalcDynamicPotentialField(gActualField, mate, enemy, nmate, nenemy);
/*
    for (i = 0; i < PX; i++) {
        for (j = 0; j < PY; j++) gActualField[i][j] = 0.0;
    }
    for (i = 0; i < PX; i++) {
	for (j = 0; j < PY; j++) {
	    if (0.0 != gDummy1[i][j]) leak1++;
	    if (0.0 != gDummy2[i][j]) leak2++;
	}
    }
    fprintf(stderr, "(%d %d) ", leak1, leak2);

    if (0 == save) {
	SavePotentialField("sfield.dat", gStaticField);
	SavePotentialField("dfield.dat", gActualField);
	SaveLocalPotentialField("lfield.dat", gActualField, &(mate[4]));
	save = 1;
    }
*/
}

void	MakeStaticPotentialField(void)
{
    int		i, j;
    Obj		tmp, mygoal, enemygoal;

    mygoal. x = 0 - 1;		/* -1 for potential field */
    mygoal. y = CY;
    enemygoal. x = X_FIELD + 1; /* +1 for potential field */
    enemygoal. y = CY;

    for (i = 0; i < PX; i++) {
	for (j = 0; j < PY; j++) {
	    gDummy1[i][j] = 0.0;
	    gDummy2[i][j] = 0.0;
	}
    }
    for (i = 0; i < PX; i++) {
	for (j = 0; j < PY; j++) {
	    tmp. x = i;
	    tmp. y = j;
	    gStaticField[i][j] =  SqrtDPotential(&mygoal, &tmp, PLUS * GOALPTL);
	    gStaticField[i][j] += SqrtDPotential(&enemygoal, &tmp, MINUS * GOALPTL);
	    if (MAXPOTENTIAL < gStaticField[i][j])  gStaticField[i][j] = MAXPOTENTIAL;
	    if (gStaticField[i][j] < -MAXPOTENTIAL) gStaticField[i][j] = -MAXPOTENTIAL;
	}
    }
}

void	CalcDynamicPotentialField(FLOAT field[PX][PY], Obj *mate, Obj *enemy, int nmate, int nenemy)
{
    int		i;

    for (i = 0; i < nmate; i++) {
	AddPlayerPotential(&(mate[i]), field, PLUS * PLAYERPTL);
    }
    for (i = 0; i < nenemy; i++) {
	AddPlayerPotential(&(enemy[i]), field, PLUS * PLAYERPTL);
    }
}

void	AddPlayerPotential(Obj *player, FLOAT field[PX][PY], int flag)
{
    int		i, j;
    int		x = (int)(player -> x), y = (int)(player -> y);
    Obj		tmp;

    if (x < 0 || X_FIELD < x || y < 0 || Y_FIELD < y) return;

    for (i = (x - PAREA < 0 ? 0 : x - PAREA); i < x; i++) {
	for (j = (y - PAREA < 0 ? 0 : y - PAREA); 
	     j < (Y_FIELD < y + PAREA ? Y_FIELD : y + PAREA); j++) {
	    tmp. x = i;
	    tmp. y = j;
	    field[i][j] += Potential(player, &tmp, flag);
	    if (MAXPOTENTIAL < field[i][j])  field[i][j] = MAXPOTENTIAL;
	    if (field[i][j] < -MAXPOTENTIAL) field[i][j] = -MAXPOTENTIAL;
	}
    }
    for (i = x; i < (X_FIELD < x + PAREA ? X_FIELD : x + PAREA); i++) {
	for (j = (y - PAREA < 0 ? 0 : y - PAREA); 
	     j < (Y_FIELD < y + PAREA ? Y_FIELD : y + PAREA); j++) {
	    tmp. x = i;
	    tmp. y = j;
	    field[i][j] += Potential(player, &tmp, flag * MINUS);
	    if (MAXPOTENTIAL < field[i][j])  field[i][j] = MAXPOTENTIAL;
	    if (field[i][j] < -MAXPOTENTIAL) field[i][j] = -MAXPOTENTIAL;
	}
    }
}

FLOAT	Potential(Obj *obj1, Obj *obj2, int flag)
{
    double	d = Distance(obj1, obj2);

    if (fabs(d) < 0.5)
	return(flag);
    else
	return(flag / d);
}

FLOAT	SqrtDPotential(Obj *obj1, Obj *obj2, int flag)
{
    double	d = Distance(obj1, obj2);

    if (fabs(d) < 0.5)
	return(flag);
    else
	return(flag / sqrt(d));
}

FLOAT	DownDirOfPotential(Obj *ball)
{
    int		i, j, a[30], tmp;
    FLOAT	x = ball -> x, y = ball -> y;
    FLOAT	p[30];

    for (i = 0;   i <= 90; i+=10) p[i/10]      = PotentialGradient(x, y, i);
    for (i = 270; i < 360; i+=10) p[i/10 - 17] = PotentialGradient(x, y, i);

    for (i = 0; i < 30; i++) a[i] = i;
    for (i = 0; i < 19 - 1; i++) {
	for (j = i + 1; j < 19; j++) {
	    if (p[a[j]] < p[a[i]]) {tmp = a[i]; a[i] = a[j]; a[j] = tmp;}
	}
    }
/*
    for (i = 0; i < 19; i++) fprintf(stderr, "%d ", (int)(p[i]*100));
    fprintf(stderr, ": %d\n", a[0]);
    fprintf(stderr, "%d ", (int)(a[0] < 10 ? a[0] * 10.0 : (a[0] + 17) * 10.0));
*/
    return(a[0] < 10 ? a[0] * 10.0 : (a[0] + 17) * 10.0);
}

FLOAT	PotentialGradient(FLOAT x, FLOAT y, int dir)
{
    int		i;
    FLOAT	p[10], nx, ny, sum = 0.0;

    p[0] = ApproxPotential(x, y);
    for (i = 1; i <= 5; i++) {
	DiredPoint(x, y, (FLOAT)dir, i * 10.0, &nx, &ny);
	if (YES == OutOfField(nx, ny))
	    p[i] = p[i - 1];
	else
	    p[i] = ApproxPotential(nx, ny);
    }
    for (i = 0; i < 5; i++) 
	sum += (p[i+1] - p[i] < 0 ? p[i+1] - p[i] : (p[i+1] - p[i])*10);
    
    return(sum);
}

FLOAT	ApproxPotential(FLOAT x, FLOAT y)
{
    int		x1 = NormalX(x), y1 = NormalY(y);
    int		x2 = NormalX(x + 1), y2 = NormalY(y + 1);
    FLOAT	r1, r2, r3, r4;

    if (x1 == x2) 
	r1 = r2 = r3 = r4 = 0.5;
    else {
	r1 = r3 = x - x1;
	r2 = r4 = x2 - x;
    }
    if (y1 == y2) {
	r1 *= 0.5; r2 *= 0.5; r3 *= 0.5; r4 *= 0.5;
    }
    else {
	r1 *= (y - y1); r2 *= (y - y1);
	r3 *= (y2 - y); r4 *= (y2 - y);
    }

    return(gActualField[x1][y1] * r1 +
	   gActualField[x2][y1] * r2 +
	   gActualField[x1][y2] * r3 +
	   gActualField[x2][y2] * r4);
}

void	CopyPotentialField(void *src, void *dst, int xsize, int ysize)
{	
    memcpy(dst, src, sizeof(FLOAT) * xsize * ysize);
}

void	SavePotentialField(char filename[], FLOAT field[PX][PY])
{
    int		i, j, mesh = 1;
    FILE	*fp;

    if (NULL == (fp = fopen(filename, "w"))) {
	fprintf(stderr, "Error: File can not open.\n");
	exit(ERR);
    }
    for (j = 0; j < PY; j += mesh) {		/* memory leak ??? */
	for (i = 0; i < PX; i += mesh) {	/* memory leak ??? */
//  for (j = CY+50; j > CY-50; j -= mesh) {	/* memory leak ??? */
//	for (i = 160; i < 240; i += mesh) {	/* memory leak ??? */
	    fprintf(fp, "%f\n", field[i][j]);	/* memory leak ??? */
	}
	fprintf(fp, "\n");
    }
    fclose(fp);
}

void	SaveLocalPotentialField(char filename[], FLOAT field[PX][PY], Obj *player)
{
    int		i, j;
    int		x = (int)(player -> x), y = (int)(player -> y);
    FILE	*fp;

    if (NULL == (fp = fopen(filename, "w"))) {
	fprintf(stderr, "Error: File can not open.\n");
	exit(ERR);
    }

    if (x < 0 || X_FIELD < x || y < 0 || Y_FIELD < y) return;

    for (j = (y - PAREA < 0 ? 0 : y - PAREA); 
	 j < (Y_FIELD < y + PAREA ? Y_FIELD : y + PAREA); j++) {
	 for (i = (x - PAREA < 0 ? 0 : x - PAREA); 
	      i < (X_FIELD < x + PAREA ? X_FIELD : x + PAREA); i++) {
	    fprintf(fp, "%f\n", field[i][j]);
	}
	fprintf(fp, "\n");
    }

    fclose(fp);
}
