/**************************************************/
/***						***/
/***	util.c					***/
/***						***/
/**************************************************/

/* gcc -ggdb -O -o robo main.c coach.c util.c shmem.c -lm */

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<math.h>

#include	"main.h"
#include	"util.h"
#include	"function.h"
#include	"flag.h"

#define         TAB             (char)0x09
#define         RET             (char)0x0a
#define         SPC             (char)0x20
#define         CMT             (char)0x23      /* '#' */

#ifndef	PI
#define		PI		3.14159265358979323846
#endif

#ifdef SIMULATE			/* simulation */
extern int	gCall;
extern FLOAT	gMovableDistance;
#else				/* real robot */
int             gCall = CALL;
FLOAT	        gMovableDistance;
#endif

CmdList         gCmdList[MAXPLAYER];

typedef unsigned long int       Ulong;

int		gBallArea;	/* ball existing area num 1 -- 12 */
int		gMateNum, gEnemyNum;
Obj		*gBall, *gMate, *gEnemy;
Obj		gMyGoal, gEnemyGoal, gFutureBall;
/*
extern Obj	ball, mate[], enemy[];
extern int	nmate, nenemy;
*/
extern int	fd;

void	f5call(int fd, CmdList *ptr);

void	NextBallPosition(Obj *ball, FLOAT time, FLOAT *x, FLOAT *y, FLOAT *dir)
{
    int		tline;
    FLOAT	tx, ty, tdir, vtime;
    Obj		tmp;

    if (YES == OutOfField(ball -> x, ball -> y)) {
        *x = ball -> x;
        *y = ball -> y;
        *dir = ball -> dir;
	return;
    }
    DiredPoint(ball -> x, ball -> y, ball -> dir, ball -> speed * time, x, y);
    *dir = ball -> dir;

    if (YES == OutOfField(*x, *y)) {
	tline = TouchLine(ball -> x, ball -> y, *x, *y, &tx, &ty, &vtime);
#ifdef	DEBUGUTIL1
	printf("tline:%d   tx:%f ty:%f  time:%f\n", 
	       tline, tx, ty, time * vtime);
#endif
	tmp. x = tx;
	tmp. y = ty;
	tmp. dir = ReflectDir(ball -> dir, tline);
	tmp. speed = ball -> speed * REFLECTSLOWDOWN;
	NextBallPosition(&tmp, time*(1.0 - vtime), &tx, &ty, &tdir);
	*x = tx;
	*y = ty;
	*dir = tdir;
#ifdef	SIMULATE
	ball -> speed *= REFLECTSLOWDOWN;
#endif
    }
}

void	NextObjPosition(Obj *obj, FLOAT time, FLOAT *x, FLOAT *y, FLOAT *dir)
{
    FLOAT	tx, ty, vtime;

    DiredPoint(obj -> x, obj -> y, obj -> dir, obj -> speed * time, x, y);
    *dir = obj -> dir;

    if (YES == OutOfField(*x, *y)) {
	TouchLine(obj -> x, obj -> y, *x, *y, &tx, &ty, &vtime);
	*x = tx;
	*y = ty;
    }
}

int	FarFromField(FLOAT x, FLOAT y)
{
    if (x < 0 - 10 || X_FIELD + 10 < x ||
	y < 0 - 10 || Y_FIELD + 10 < y) return(YES);
    return(NO);
}

int	OutOfField(FLOAT x, FLOAT y)
{
    if (x < 0 || X_FIELD < x || y < 0 || Y_FIELD < y) return(YES);
    return(NO);
}

int	OutOfX(FLOAT x)
{
    if (x < 0 || X_FIELD < x) return(YES);
    return(NO);
}

int	OutOfY(FLOAT y)
{
    if (y < 0 || Y_FIELD < y) return(YES);
    return(NO);
}

FLOAT	NormalX(FLOAT x)
{
    if (x < 0) return(0.0);
    if (X_FIELD < x) return((FLOAT)X_FIELD);
    return(x);
}

FLOAT	NormalY(FLOAT y)
{
    if (y < 0) return(0.0);
    if (Y_FIELD < y) return((FLOAT)Y_FIELD);
    return(y);
}

void	RotateXY(FLOAT x, FLOAT y, FLOAT degree, FLOAT cx, FLOAT cy, FLOAT *nx, FLOAT *ny)
{
    FLOAT	theta = D2R(degree);

    *nx = (x - cx) * cos(theta) - (y - cy) * sin(theta) + cx;
    *ny = (x - cx) * sin(theta) + (y - cy) * cos(theta) + cy;
}

FLOAT	D2R(FLOAT degree)
/* Degree 2 Radian */
{
    return((FLOAT)PI * degree / 180.0);
}

FLOAT	R2D(FLOAT radian)
/* Radian 2 Degree*/
{
    return(radian / PI * 180.0);
}

int	XYArea(FLOAT x, FLOAT y)
{
    if (x < 0           && y < 0)		return(1);
    if (NO == OutOfX(x) && y < 0)		return(2);
    if (X_FIELD < x     && y < 0)		return(3);
    if (x < 0           && NO == OutOfY(y))	return(4);
    if (NO == OutOfX(x) && NO == OutOfY(y))	return(0); /* inside */
    if (X_FIELD < x     && NO == OutOfY(y))	return(5);
    if (x < 0           && Y_FIELD < y)		return(6);
    if (NO == OutOfX(x) && Y_FIELD < y)		return(7);
/*  if (X_FIELD < x     && Y_FIELD < y)*/	return(8);
}

int	TouchLine(FLOAT px, FLOAT py, FLOAT x, FLOAT y, FLOAT *tx, FLOAT *ty, FLOAT *vtime)
{
    int		code;

    code = XYArea(x, y);

    switch (code) {
    case 2: 
	*tx = px - (px - x) * py / (py - y);
	*ty = 0;
	*vtime = fabs(py / (py - y));
	return(LEFTSIDE);
        break;
    case 4: 
	*tx = 0;
	*ty = py - (py - y) * px / (px - x);
	*vtime = fabs(px / (px - x));
	return(MYSIDE);
	break;
    case 5: 
	*tx = X_FIELD;
	*ty = py + (py - y) * (X_FIELD - px) / (px - x);
	*vtime = fabs((X_FIELD - px) / (px - x));
	return(ENEMYSIDE);
	break;
    case 7: 
	*tx = px + (px - x) * (Y_FIELD - py) / (py - y);
	*ty = Y_FIELD;
	*vtime = fabs((Y_FIELD - py) / (py - y));
	return(RIGHTSIDE);
	break;
    case 1: 
	*ty = py - (py - y) * px / (px - x);
	if (0 < *ty) {
	    *tx = 0;
	    *vtime = fabs(px / (px - x));
	    return(MYSIDE);
	}
	else {
	    *tx = px - (px - x) * py / (py - y);
	    *ty = 0;
	    *vtime = fabs(py / (py - y));
	    return(LEFTSIDE);
	}
	break;
    case 3:
	*ty = py + (py - y) * (X_FIELD - px) / (px - x);
	if (0 < *ty) {
	    *tx = X_FIELD;
	    *vtime = fabs((X_FIELD - px) / (px - x));
	    return(ENEMYSIDE);
	}
	else {
	    *tx = px - (px - x) * py / (py - y);
	    *ty = 0;
	    *vtime = fabs(py / (py - y));
	    return(LEFTSIDE);
	}
	break;
    case 6:
	*ty = py - (py - y) * px / (px - x);
	if (*ty < Y_FIELD) {
	    *tx = 0;
	    *vtime = fabs(px / (px - x));
	    return(MYSIDE);
	}
	else {
	    *tx = px + (px - x) * (Y_FIELD - py) / (py - y);
	    *ty = Y_FIELD;
	    *vtime = fabs((Y_FIELD - py) / (py - y));
	    return(RIGHTSIDE);
	}
	break;
    case 8:
	*ty = py + (py - y) * (X_FIELD - px) / (px - x);
	if (*ty < Y_FIELD) {
	    *tx = X_FIELD;
	    *vtime = fabs((X_FIELD - px) / (px - x));
	    return(ENEMYSIDE);
	}
	else {
	    *tx = px + (px - x) * (Y_FIELD - py) / (py - y);
	    *ty = Y_FIELD;
	    *vtime = fabs((Y_FIELD - py) / (py - y));
	    return(RIGHTSIDE);
	}
	break;
    default:
	break;
    }
    return(0);	/* never reach here */
}

FLOAT	ReflectDir(FLOAT dir, int linecode)
{
    if (LEFTSIDE == linecode || RIGHTSIDE == linecode)
	return(NormalDir(-dir));
    else
	return(NormalDir(180 - dir));
}

FLOAT	NormalDir(FLOAT dir)	/* 0 -- 360 */
{
    if (dir < 0) return(dir + 360);
    if (360 <= dir) return(dir - 360);
    return(dir);
}

FLOAT	Dir180(FLOAT dir)		/* -180 -- 180 */
{
    if (dir <= -180) return(dir + 360);
    if (180.0 < dir) return(dir - 360);
    return(dir);
}

FLOAT	SqrDistance(Obj *obj1, Obj *obj2)
{
    return((obj1 -> x - obj2 -> x) * (obj1 -> x - obj2 -> x) +
	   (obj1 -> y - obj2 -> y) * (obj1 -> y - obj2 -> y));
}

FLOAT	SqrDistanceFromXY(FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2)
{
    return((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
}

FLOAT	Distance(Obj *obj1, Obj *obj2)
{
    return(DistanceFromXY(obj1 -> x, obj1 -> y, obj2 -> x, obj2 -> y));
}

FLOAT	DistanceFromXY(FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2)
{
    return((FLOAT)sqrt((double)(x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1)));
}

FLOAT	Dir(Obj *obj1, Obj *obj2)
{
    return(DirFromXY(obj1 -> x, obj1 ->y, obj2 -> x, obj2 ->y));
}

FLOAT	DirFromXY(FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2)
{
    if (fabs(x1 - x2) < SMALLNUM) {
	if (y1 < y2) return(90.0);
	else	     return(270.0);
    }
    if (x1 < x2)
	return(NormalDir(R2D(atan((y2 - y1) / (x2 - x1)))));
    else
	return(NormalDir(180.0 - R2D(atan((y2 - y1) / -(x2 - x1)))));
}

void	DiredPoint(FLOAT x, FLOAT y, FLOAT dir, FLOAT distance, FLOAT *nx, FLOAT *ny)
{
    *nx = x + (cos(D2R(dir)) * distance);
    *ny = y + (sin(D2R(dir)) * distance);
}

int	XPoint4Objs(Obj *obj1, Obj *obj2, Obj *obj3, Obj *obj4, FLOAT *cx, FLOAT *cy)
{
    return(XPoint2Lines(obj1 -> x, obj1 -> y, obj2 -> x, obj2 -> y,
			obj3 -> x, obj3 -> y, obj4 -> x, obj4 -> y, cx, cy));
}

int	XPoint2Lines(FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2, FLOAT x3, FLOAT y3, FLOAT x4, FLOAT y4, FLOAT *cx, FLOAT *cy)
{
    int         linecode1, linecode2;
    FLOAT       a1, b1, a2, b2;

    linecode1 = LineParam(x1, y1, x2, y2, &a1, &b1);
    linecode2 = LineParam(x3, y3, x4, y4, &a2, &b2);

    if (VERTICAL == linecode1 && VERTICAL == linecode2) {
	*cx = 0;
	*cy = 0;
	return(ERR);
    }
    if (VERTICAL == linecode1 && HORIZONTAL == linecode2) {
	*cx = x1;
	*cy = y3;
	return(NORMAL);
    }
    if (VERTICAL == linecode1 && NORMAL == linecode2) {
	*cx = x1;
	*cy = a2 * x1 + b2;
	return(NORMAL);
    }
    if (HORIZONTAL == linecode1 && VERTICAL == linecode2) {
	*cx = x3;
	*cy = y1;
	return(NORMAL);
    }
    if (HORIZONTAL == linecode1 && HORIZONTAL == linecode2) {
	*cx = 0;
	*cy = 0;
	return(ERR);
    }
    if (HORIZONTAL == linecode1 && NORMAL == linecode2) {
	*cx = (y1 - b2) / a2;
	*cy = y1;
	return(NORMAL);
    }
    if (NORMAL == linecode1 && VERTICAL == linecode2) {
	*cx = x3;
        *cy = a1 * x3 + b1;
        return(NORMAL);
    }
    if (NORMAL == linecode1 && HORIZONTAL == linecode2) {
	*cx = (y3 - b1) / a1;
	*cy = y3;
	return(NORMAL);
    }
    if (NORMAL == linecode1 && NORMAL == linecode2) {
	if (fabs(a1 - a2) < SMALLNUM) {
	    *cx = 0;
	    *cy = 0;
	    return(ERR);
	}
	else {
	    *cx = -(b1 - b2) / (a1 - a2);
	    *cy = a1 * (*cx) + b1;
	    return(NORMAL);
	}
    }
    return(ERR);	/* never reach here */
}

int	XPointDiredPoint2Line(FLOAT x1, FLOAT y1, FLOAT dir, FLOAT x2, FLOAT y2, FLOAT x3, FLOAT y3, FLOAT *cx, FLOAT *cy)
{
    FLOAT	x, y;

    DiredPoint(x1, y1, dir, 10.0, &x, &y);
    return(XPoint2Lines(x1, y1, x, y, x2, y2, x3, y3, cx, cy));
}

void	XPointPoint2Line(FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2, FLOAT x3, FLOAT y3, FLOAT *cx, FLOAT *cy)
{
    int		linecode;
    FLOAT	a1, b1, a2, b2;

    linecode = LineParam(x2, y2, x3, y3, &a1, &b1);

    switch (linecode) {
    case VERTICAL:
/*	distance = fabs(x1 - x2);*/
	*cx = x2;
	*cy = y1;
	break;
    case HORIZONTAL:
/*	distance = fabs(y1 - y2);*/
	*cx = x1;
	*cy = y2;
	break;
    case NORMAL:
	a2 = -1.0 / a1;
	b2 = y1 - a2 * x1;
	*cx = -(b1 - b2) / (a1 - a2);
	*cy = a1 * (*cx) + b1;
	break;
    default:
	break;
    }
}

int	LineParam(FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2, FLOAT *a, FLOAT *b)
{
    if (0.0 == (x1 - x2)) {
	*a = BIGNUM;
	*b = x1;
	return(VERTICAL);
    }
    else {
	*a = (y2 - y1) / (x2 - x1);
	if (BIGNUM < fabs(*a)) {
//	    *a = BIGNUM;
	    *b = x1;
	    return(VERTICAL);
	}
	else if (fabs(*a) < SMALLNUM) {
//	    *a = 0.0;
	    *b = y1;
	    return(HORIZONTAL);
	}
	else {
	    *b = y1 - *a * x1;
	    return(NORMAL);
	}
    }
}

int	FieldStatus(int ballarea, int area[])
{
    if (ballarea <= 2)		return(DANGER);
    else if (ballarea <= 6)	return(CARE);
    else if (ballarea <= 9)	return(NORMAL);
    else			return(SAFE);
}

void	AllObjArea(int ary[])
{
    int		i;

    for (i = 0; i <= 12; i++) ary[i] = 0;

    ary[BallArea()]++;
    for (i = 0; i < gMateNum; i++) ary[ObjArea(&(gMate[i]))]++;
    for (i = 0; i < gEnemyNum; i++) ary[ObjArea(&(gEnemy[i]))]++;
}

int	BallArea(void)
{
    return(ObjArea(gBall));
}

int	ObjArea(Obj *obj)
{
    if (obj -> x < 20) {
	if (obj -> y < Y_FIELD / 3)     return(3);
	if (Y_FIELD * 2 / 3 < obj -> y) return(4);
	return(1);
    }
    if (obj -> x < X_FIELD / 2) {
	if (obj -> y < 20)	     return(5);
	if (Y_FIELD - 20 < obj -> y) return(6);
	return(2);
    }
    if (obj -> x < X_FIELD - 20) {
	if (obj -> y < 20)	     return(8);
	if (Y_FIELD - 20 < obj -> y) return(9);
	return(7);
    }
    else {
	if (obj -> y < Y_FIELD / 3)     return(11);
	if (Y_FIELD * 2 / 3 < obj -> y) return(12);
	return(10);
    }
}

int	PenaltyArea(Obj *obj, int side)
{
    return(PenaltyAreaXY(obj -> x, obj -> y, side));
}

int	PenaltyAreaXY(FLOAT x, FLOAT y, int side)
{
    if (MYSIDE == side) {
	if (x < PENALTYX && PENALTYY1 < y && y < PENALTYY2)
	    return(YES);
	else
	    return(NO);
    }
    else /* if (ENEMYSIDE == side) */ {
	if ((X_FIELD - PENALTYX) < x && PENALTYY1 < y && y < PENALTYY2)
	    return(YES);
	else
	    return(NO);
    }
}

void	ShootPosition(Obj *player, Obj *ball, Obj *enemy, int nenemy, FLOAT *x, FLOAT *y)
{
    int		tline, target;
    FLOAT	tx, ty, vtime, d;
    Obj         *gk;
/*
    if (160 < ball -> dir && ball -> dir < 200) {
	DiredPoint(ball -> x, ball -> y, ball -> dir, 10.0, &tx, &ty);
	XPointPoint2Line(player -> x, player -> y, ball -> x, ball -> y, 
			 tx, ty, x, y);
	if (DistanceFromXY(player -> x, player -> y, *x, *y) < gMovableDistance) 
	    return;
    }
*/
    gk = &(enemy[NearestObjPlayer(enemy, &gEnemyGoal, nenemy, &d)]);
    if (gk -> y < CY) target = 20;
    else              target = -20;

    DiredPoint(ball -> x, ball -> y, 
	       DirFromXY(gEnemyGoal. x, gEnemyGoal. y + target, ball -> x, ball -> y), 
	       10.0, x, y);

    if (YES == OutOfField(*x, *y)) {
        tline = TouchLine(gEnemyGoal. x, gEnemyGoal. y + target, *x, *y, &tx, &ty, &vtime);
	*x = tx;
	*y = ty;
    }
}

void	OfPosition(Obj *player, Obj *obj, FLOAT *x, FLOAT *y)
{
    int		tline;
    FLOAT	tx, ty, vtime;

    if (160 < obj -> dir && obj -> dir < 200 && player -> x < obj -> x) {
	DiredPoint(obj -> x, obj -> y, obj -> dir, 10.0, &tx, &ty);
	XPointPoint2Line(player -> x, player -> y, obj -> x, obj -> y, 
			 tx, ty, x, y);
	if (DistanceFromXY(player -> x, player -> y, *x, *y) < gMovableDistance) 
	    return;
    }

    DiredPoint(obj -> x, obj -> y, 
	       DirFromXY(gEnemyGoal. x, gEnemyGoal. y, obj -> x, obj -> y), 
	       10.0, x, y);

    if (YES == OutOfField(*x, *y)) {
        tline = TouchLine(gEnemyGoal. x, gEnemyGoal. y, *x, *y, &tx, &ty, &vtime);
	*x = tx;
	*y = ty;
    }
}

void	DfPosition(Obj *player, Obj *obj, FLOAT *x, FLOAT *y)
{
    DiredPoint(obj -> x, obj -> y, 
	       DirFromXY(obj -> x, obj -> y, gMyGoal. x, gMyGoal. y), 
	       10.0, x, y);
}

void	DfWaitPosition(Obj *player, Obj *obj, FLOAT *x, FLOAT *y)
{
    DiredPoint(gMyGoal. x, gMyGoal. y, 
	       DirFromXY(gMyGoal. x, gMyGoal. y, obj -> x, obj -> y), 
	       30, x, y);
/*
    XPointPoint2Line(player -> x, player -> y, obj -> x, obj -> y, 
		     gMyGoal. x, gMyGoal. y, x, y);
*/
}

FLOAT	BallDir(Obj *obj)
{
    return(DirFromXY(obj -> x, obj -> y, gBall -> x, gBall -> y));
}

FLOAT	FutureBallDir(Obj *obj)
{
    return(DirFromXY(obj -> x, obj -> y, gFutureBall. x, gFutureBall. y));
}

FLOAT	BallDistance(Obj *obj)
{
    return(DistanceFromXY(obj -> x, obj -> y, gBall -> x, gBall -> y));
}

FLOAT	FutureBallDistance(Obj *obj)
{
    return(DistanceFromXY(obj -> x, obj -> y, gFutureBall. x, gFutureBall. y));
}

void	OrderBallPlayer(Obj *player, int num, int a[], FLOAT d[])
{
    int		i, j, tmp;
    FLOAT	ftmp;

    for (i = 0; i < num; i++) {
	a[i] = player[i]. id;
	d[i] = BallDistance(&(player[i]));
    }
    for (i = 0; i < num - 1; i++) {
	for (j = i + 1; j < num; j++) {
	    if (d[j] < d[i]) {
		tmp = a[i]; a[i] = a[j]; a[j] = tmp;
		ftmp= d[i]; d[i] = d[j]; d[j] =ftmp;
	    }
	}
    }
    if (gBall -> x < player[a[0]]. x - 0.0 && player[a[1]]. x < gBall -> x) {
        tmp = a[0]; a[0] = a[1]; a[1] = tmp;
	ftmp= d[0]; d[0] = d[1]; d[1] =ftmp;
    }
}

void	OrderFutureBallPlayer(Obj *player, int num, int a[], FLOAT d[])
{
    int		i, j, tmp;
    FLOAT	ftmp;

    for (i = 0; i < num; i++) {
	a[i] = player[i]. id; 
	d[i] = FutureBallDistance(&(player[i]));
    }
    for (i = 0; i < num - 1; i++) {
	for (j = i + 1; j < num; j++) {
	    if (d[j] < d[i]) {
		tmp = a[i]; a[i] = a[j]; a[j] = tmp;
		ftmp= d[i]; d[i] = d[j]; d[j] =ftmp;
	    }
	}
    }
    if (gFutureBall. x < player[a[0]]. x - 0.0 && player[a[1]]. x < gFutureBall. x) {
        tmp = a[0]; a[0] = a[1]; a[1] = tmp;
	ftmp= d[0]; d[0] = d[1]; d[1] =ftmp;
    }
}

int     WhoisBestKicker(Obj *player, FLOAT dist[], FLOAT x, FLOAT y)
{
    int         i, flag[MAXPLAYER] = {NO, YES, YES, YES, YES};
    int         min;
    FLOAT       th = 50, dir[MAXPLAYER] = {1000, 1000, 1000, 1000, 1000};
    Obj         tmp;

    tmp. x = x;
    tmp. y = y;
    for (i = 1; i < MAXPLAYER; i++) {
        if (th < dist[i]) flag[i] = NO;
	if (YES == InsideAllRobots(&(player[i]), &tmp)) flag[i] = NO;
    }
    for (i = 1; i < MAXPLAYER; i++) {
      if (YES == flag[i]) {
	dir[i] = fabs(Dir(gBall, &gEnemyGoal) - Dir(&(player[i]), &gEnemyGoal));
      }
    }
    min = 1;
    for (i = 2; i < MAXPLAYER; i++) if (dir[i] < dir[min]) min = i;

    if (dir[min] < 30) return(min);
    else               return(-1);
}

Obj*	NearestBallRobot(Obj *mate, Obj *enemy, int nmate, int nenemy, FLOAT *distance)
{
    int		maten, enemyn;
    FLOAT	mated, enemyd;
    Obj		*obj;

    maten = NearestBallMate(mate, nmate, &mated);
    enemyn = NearestBallEnemy(enemy, nenemy, &enemyd);
    if (mated < enemyd) {obj = &(mate[maten]); *distance = mated;}
    else                {obj = &(enemy[enemyn]); *distance = enemyd;}

    return(obj);
}

int	NearestObjPlayer(Obj *player, Obj *obj, int num, FLOAT *distance)
{
    int		i, nearest = 0;
    FLOAT	tmp;

#ifdef  SIMULATE
    if (0 == num) {*distance = BIGNUM; return(0);}
#endif
    *distance = Distance(&(player[0]), obj);
    for (i = 1; i < num; i++) {
	tmp = Distance(&(player[i]), obj);
	if (tmp < *distance) {*distance = tmp; nearest = i;}
    }
    return(nearest);
}

int	NearestBallMate(Obj *obj, int num, FLOAT *distance)
{
    int		i, nearest = 0;
    FLOAT	tmp;

#ifdef  SIMULATE
    if (0 == num) {*distance = BIGNUM; return(0);}
#endif
    *distance = BallDistance(&(obj[0]));
    for (i = 1; i < num; i++) {
	tmp = BallDistance(&(obj[i]));
	if (tmp < *distance) {*distance = tmp; nearest = i;}
    }
    return(nearest);
}

int	NearestFutureBallMate(Obj *obj, int num, FLOAT *distance)
{
    int		i, nearest = 0;
    FLOAT	tmp;

#ifdef  SIMULATE
    if (0 == num) {*distance = BIGNUM; return(0);}
#endif
    *distance = FutureBallDistance(&(obj[0]));
    for (i = 1; i < num; i++) {
	tmp = FutureBallDistance(&(obj[i]));
	if (tmp < *distance) {*distance = tmp; nearest = i;}
    }
    return(nearest);
}

int	NearestBallEnemy(Obj *obj, int num, FLOAT *distance)
{
    return(NearestBallMate(obj, num, distance));
}

int	NearestFutureBallEnemy(Obj *obj, int num, FLOAT *distance)
{
    return(NearestFutureBallMate(obj, num, distance));
}

int	AroundBall(FLOAT r, int *matenum, int *enemynum)
{
    int         i;

    *matenum = 0;
    for (i = 0; i < gMateNum; i++) {
        if (BallDistance(&(gMate[i])) < r) (*matenum)++;
    }
    *enemynum = 0;
    for (i = 0; i < gEnemyNum; i++) {
	if (BallDistance(&(gEnemy[i])) < r) (*enemynum)++;
    }
    return(*matenum + *enemynum);
}

int     InsideAllRobots(Obj *player, Obj *point)
{
  int i;

  for (i = 0; i < gMateNum; i++) {
    if (&(gMate[i]) == player) continue;
    if (YES == Inside2Objs(&(gMate[i]), player, point)) return(YES);
  }
  for (i = 0; i < gEnemyNum; i++) {
    if (YES == Inside2Objs(&(gEnemy[i]), player, point)) return(YES);
  }
  return(NO);
}

int	Inside2Objs(Obj *target, Obj *obj1, Obj *obj2)
{
    FLOAT	distance, x, y, th;

    XPointPoint2Line(target -> x, target -> y, obj1 -> x, obj1 -> y,
		     obj2 -> x, obj2 -> y, &x, &y);
    distance = DistanceFromXY(target -> x, target -> y, x, y);
    th = gBall==target ? ROBOTSIZE + BALLSIZE + 1.0 : ROBOTSIZE * 2 + 2.0;

    if (th < distance) 
	return(NO);
    else 
	return(InsideXY(x, y, obj1 -> x, obj1 -> y, obj2 -> x, obj2 -> y));
}

int	InsideXY(FLOAT tx, FLOAT ty, FLOAT x1, FLOAT y1, FLOAT x2, FLOAT y2)
{
    FLOAT	maxx, maxy, minx, miny;

    if (x1 < x2) {minx = x1; maxx = x2;}
    else	 {minx = x2; maxx = x1;}
    if (y1 < y2) {miny = y1; maxy = y2;}
    else	 {miny = y2; maxy = y1;}

    if (minx <= tx && tx <= maxx && miny <= ty && ty <= maxy)
	return(YES);
    else
	return(NO);
}

void	AvoidCollisionBall(Obj *player, FLOAT *x, FLOAT *y)
{
    FLOAT	dir = 0.0;

    AvoidCollisionObj(player, gBall, x, y, &dir);
}	   

void	AvoidCollisionAll(Obj *player, FLOAT *x, FLOAT *y)
{
    int		i, count1 = 0, count2 = 0;
    FLOAT	x1, y1, orgx = *x, orgy = *y;
    FLOAT	dir = 0.0, th = 40;
    FLOAT	db, de[MAXPLAYER*2] = {300, 300, 300, 300, 300, 300, 300, 300, 300, 300};

    db = Distance(player, gBall);
    for (i = 0; i < gMateNum; i++) {
        if (gMate[i]. id == player -> id) continue;
        de[i] = Distance(player, &(gMate[i]));
    }
    for (i = 0; i < gEnemyNum; i++) de[i+5] = Distance(player, &(gEnemy[i]));

    if (*y < player -> y) dir = 90; 
    else		  dir = -90;

    while (1) {
        if (10 < count1++) break;
        if (db < th && (gBall -> x - ROBOTSIZE) < player -> x && 
	    YES == AvoidCollisionObj(player, gBall, x, y, &dir)) 
	    continue;
        if (de[0] < th && YES == AvoidCollisionObj(player, &(gMate[0]), x, y, &dir)) 
	    continue;
        if (de[1] < th && YES == AvoidCollisionObj(player, &(gMate[1]), x, y, &dir)) 
	    continue;
        if (de[2] < th && YES == AvoidCollisionObj(player, &(gMate[2]), x, y, &dir)) 
	    continue;
        if (de[3] < th && YES == AvoidCollisionObj(player, &(gMate[3]), x, y, &dir)) 
	    continue;
        if (de[4] < th && YES == AvoidCollisionObj(player, &(gMate[4]), x, y, &dir)) 
	    continue;
        if (de[5] < th && YES == AvoidCollisionObj(player, &(gEnemy[0]), x, y, &dir)) 
	    continue;
        if (de[6] < th && YES == AvoidCollisionObj(player, &(gEnemy[1]), x, y, &dir)) 
	    continue;
        if (de[7] < th && YES == AvoidCollisionObj(player, &(gEnemy[2]), x, y, &dir)) 
	    continue;
        if (de[8] < th && YES == AvoidCollisionObj(player, &(gEnemy[3]), x, y, &dir)) 
	    continue;
        if (de[9] < th && YES == AvoidCollisionObj(player, &(gEnemy[4]), x, y, &dir)) 
	    continue;
	break;
    }

    x1 = *x; y1 = *y;

    if (1 < count1) {
        dir *= -1; *x = orgx; *y = orgy;
    while (1) {
        if (count1 < count2++) break;
        if (db < th && (gBall -> x - ROBOTSIZE) < player -> x && 
	    YES == AvoidCollisionObj(player, gBall, x, y, &dir)) 
	    continue;
        if (de[0] < th && YES == AvoidCollisionObj(player, &(gMate[0]), x, y, &dir)) 
	    continue;
        if (de[1] < th && YES == AvoidCollisionObj(player, &(gMate[1]), x, y, &dir)) 
	    continue;
        if (de[2] < th && YES == AvoidCollisionObj(player, &(gMate[2]), x, y, &dir)) 
	    continue;
        if (de[3] < th && YES == AvoidCollisionObj(player, &(gMate[3]), x, y, &dir)) 
	    continue;
        if (de[4] < th && YES == AvoidCollisionObj(player, &(gMate[4]), x, y, &dir)) 
	    continue;
        if (de[5] < th && YES == AvoidCollisionObj(player, &(gEnemy[0]), x, y, &dir)) 
	    continue;
        if (de[6] < th && YES == AvoidCollisionObj(player, &(gEnemy[1]), x, y, &dir)) 
	    continue;
        if (de[7] < th && YES == AvoidCollisionObj(player, &(gEnemy[2]), x, y, &dir)) 
	    continue;
        if (de[8] < th && YES == AvoidCollisionObj(player, &(gEnemy[3]), x, y, &dir)) 
	    continue;
        if (de[9] < th && YES == AvoidCollisionObj(player, &(gEnemy[4]), x, y, &dir)) 
	    continue;
	break;
    }
    }

    if (count1 < count2) {*x = x1; *y = y1;}
    else count1 = count2;

    for (i = 0; i < gEnemyNum; i++) {
        if (DistanceFromXY(*x, *y, gEnemy[i]. x, gEnemy[i]. y) < ROBOTSIZE) {
	    DiredPoint(player -> x, player -> y, DirFromXY(player -> x, player -> y, *x, *y),
		       DistanceFromXY(player -> x, player -> y, *x, *y) - ROBOTSIZE, x, y);
	    break;
	}
    }
}	   

int	AvoidCollisionObj(Obj *player, Obj *obs, FLOAT *x, FLOAT *y, FLOAT *dir)
{
    int		r;
    FLOAT	d;
    Obj		tmp;

    tmp. x = *x;
    tmp. y = *y;
    r = Inside2Objs(obs, player, &tmp);
    if (YES == r) {
	if (0.0 == *dir) {
	    if (obs -> y < player -> y)	*dir = -90;
	    else 			*dir = 90;
	}
        d = DirFromXY(player -> x, player -> y, *x, *y);
	DiredPoint(obs -> x, obs -> y, NormalDir(d + *dir), 
		   gBall==obs?
		   (ROBOTSIZE+BALLSIZE)*AVOIDCOLLISIONPARAM:
		   (ROBOTSIZE*2)*AVOIDCOLLISIONPARAM,
		   x, y);
    }
    return(r);
}

void	InitVar(Obj *ball, Obj *mate, Obj *enemy, int nmate, int nenemy)
{
    int		i;

    gMateNum = nmate;
    gEnemyNum = nenemy;
    gBall = ball;
    gMate = mate;
    gEnemy = enemy;

    gMovableDistance = MOVABLEDISTANCE;

    for (i = 0; i < MAXPLAYER; i++) {
	gMate[i]. id = gEnemy[i]. id = i;
	gMate[i]. radio = gEnemy[i]. radio = i;
	gMate[i]. move = MOVE;
    }
    gMyGoal. x = 0 - 1;		/* -1 for potential field */
    gMyGoal. y = CY;
    gEnemyGoal. x = X_FIELD + 1; /* +1 for potential field */
    gEnemyGoal. y = CY;

    for (i = 0; i < MAXPLAYER; i++) {
	gCmdList[i]. cmd = 'x';
	gCmdList[i]. para[0] = 0;
	gCmdList[i]. para[1] = 0;
	gCmdList[i]. para[2] = 0;
    }
}

void	ReadInitData(char fname[], Obj *ball, Obj *mate, Obj *enemy, int *nmate, int *nenemy)
{
    int		i, token;
    char	str[BUFSIZ];
    FLOAT	ary[10];
    FILE	*fp;

    if ((FILE *)NULL == (fp = fopen(fname, "r"))) {
	fprintf(stderr, "Error: [%s] can not open.\n", fname);
	exit(ERR);
    }

    fgets(str, BUFSIZ, fp); token = Token(str, ary); SetObjVal(ball, ary);
    ball -> speed = ary[5];

    *nmate = 0;
    for (i = 0; i < MAXPLAYER; i++) {
	fgets(str, BUFSIZ, fp);
	token = Token(str, ary);
	if (YES == SetObjVal(&(mate[*nmate]), ary)) (*nmate)++;
    }
    ary[0] = -1;
    for (i = *nmate; i < MAXPLAYER; i++) SetObjVal(&(mate[i]), ary);

    *nenemy = 0;
    for (i = 0; i < MAXPLAYER; i++) {
	fgets(str, BUFSIZ, fp);
	token = Token(str, ary);
	if (YES == SetObjVal(&(enemy[*nenemy]), ary)) (*nenemy)++;
    }
    ary[0] = -1;
    for (i = *nenemy; i < MAXPLAYER; i++) SetObjVal(&(enemy[i]), ary);

    fclose(fp);
}

int	SetObjVal(Obj *obj, FLOAT ary[])
{
    if (ary[0] < 0.0) {
	obj -> x = -1000;
	obj -> y = -1000;
	obj -> dir = 0;
	obj -> px = 0;
	obj -> py = 0;
	obj -> speed = 0;
	obj -> move = STOP;
	return(NO);
    }
    else {
	obj -> x = ary[0];
	obj -> y = ary[1];
	obj -> dir = ary[2];
	obj -> px = 0;
	obj -> py = 0;
	obj -> speed = 0;
	obj -> move = MOVE;
	return(YES);
    }
#ifdef	DEBUGUTIL1
    printf("%f %f %f %f %f %f\n", obj -> x, obj -> y, obj -> dir,
	   obj -> px, obj -> py, obj -> speed);
#endif
}

void	GoArc(Obj *obj, int rotate, FLOAT x, FLOAT y, FLOAT dir)
{
    FLOAT	mx, my, cx, cy, d;

    mx = (obj -> x + x) / 2;
    my = (obj -> y + y) / 2;
    d = DirFromXY(obj -> x, obj -> y, x, y);
    DiredPoint(mx, my, (ROTATELEFT==rotate?NormalDir(d+90):NormalDir(d-90)), 
	       10.0, &cx, &cy);
    GoXY(obj, cx, cy);
}

/* Old version
void	GoArc(Obj *obj, int rotate, FLOAT x, FLOAT y, FLOAT dir)
{
    int		r, len;
    FLOAT	dist, degree;

    dist = DistanceFromXY(obj -> x, obj -> y, x, y);
    r = dist * 10 * 0.8;
    len = dist * 10 * 1.03;
    degree = NormalDir(dir + (ROTATELEFT==rotate?38:-38));

    if (5 < fabs(Dir180(obj -> dir - degree))) 
	TurnAbsDir(obj, degree);
    else
	Setarc(fd, obj -> id, rotate, r, len);
}
*/

void	GoXY(Obj *obj, FLOAT x, FLOAT y)
{
#ifdef	AVOIDBALLONLY
    AvoidCollisionBall(obj, &x, &y);
#endif
#ifdef	AVOIDALL
    AvoidCollisionAll(obj, &x, &y);
#endif
/*
    if (YES == FarFromField(obj -> x, obj -> y))
        fprintf(stderr, "%d,%d(%d,%d) ", (int)obj -> x, (int)obj -> y, (int)x, (int)y);
*/
    GoAbsDir(obj, DirFromXY(obj -> x, obj -> y, x, y), 
	     DistanceFromXY(obj -> x, obj -> y, x, y));
}

void	GoXYThru(Obj *obj, FLOAT x, FLOAT y)
{
#ifdef	AVOIDBALLONLY
    AvoidCollisionBall(obj, &x, &y);
#endif
#ifdef	AVOIDALL
    AvoidCollisionAll(obj, &x, &y);
#endif
/*
    if (YES == FarFromField(obj -> x, obj -> y))
        fprintf(stderr, "%d,%d(%d,%d) ", (int)obj -> x, (int)obj -> y, (int)x, (int)y);
*/
    GoAbsDir(obj, DirFromXY(obj -> x, obj -> y, x, y), 
	     DistanceFromXY(obj -> x, obj -> y, x, y) + 10.0);
}

void	GoThru(Obj *obj, int distance)
{
    Setgo_dist_mm(fd, obj -> id, distance);
}

void	GoAbsDir(Obj *obj, FLOAT dir, FLOAT distance)
{
    GoRelDir(obj, Dir180(dir - obj -> dir), distance);
}

void	GoRelDir(Obj *obj, FLOAT dir, FLOAT distance) /* accept -180 -- 180 */
{
    dir *= ROTATERATE;
#ifdef	STOPBEFORERG
    stop_robo(fd, obj -> id);
#endif
    if (135 < dir) { dir -= 180.0; distance *= -1; }
    else if (dir < -135){ dir += 180.0; distance *= -1; }

    if (fabs(dir) < IGNOREDEG) {Go(obj, distance); return;}

    if (fabs(distance) < IGNOREDIS) {TurnRelDir(obj, dir); return;}

    if (fabs(distance) < gMovableDistance) {
	Setrotate_and_go(fd, obj -> id, (int)dir, (int)(distance * 10));
    }
    else {
	Setrotate_and_go(fd, obj -> id, (int)dir, 
			 (int)((0<distance?1:-1) * gMovableDistance * 10));
    }
}

void	Go(Obj *obj, FLOAT distance)
{
#ifdef	STOPBEFOREG
    stop_robo(fd, obj -> id);
#endif
    if (fabs(distance) < IGNOREDIS)
        Setgo_dist_mm(fd, obj -> id, 0);
    else if (fabs(distance) < gMovableDistance) 
	Setgo_dist_mm(fd, obj -> id, (int)(distance * 10));
    else
	Setgo_dist_mm(fd, obj -> id, (int)((0<distance?1:-1) * gMovableDistance * 10));
}

void	TurnRelDir(Obj *obj, FLOAT dir)
{
    dir *= ROTATERATE;
#ifdef	STOPBEFORER
    stop_robo(fd, obj -> id);
#endif
    if (fabs(dir) < IGNOREDEG) 
        Setrotate_theta(fd, obj -> id, 0);
    else
        Setrotate_theta(fd, obj -> id, (int)dir);
}

void	TurnAbsDir(Obj *obj, FLOAT dir)
{
    TurnRelDir(obj, Dir180(dir - obj -> dir));
}

void	Nop(Obj *obj)
{
    Setnop(fd, obj -> id);
}

void	Stay(Obj *obj)
{
    Setgo_dist_mm(fd, obj -> id, 0);
}

void	StopRobot(void)
{
    int		i;

    for (i = 0; i < MAXPLAYER; i++) Setgo_dist_mm(fd, i, 0);
    SendCmdList();
}

void	Setgo_dist_mm(int fd, int no, int n)
{
    gCmdList[no]. cmd = 'g';
    gCmdList[no]. para[0] = (short)n;
    gCmdList[no]. para[1] = (short)0;
    gCmdList[no]. para[2] = (short)DASHSPEED; // originally 0 (=default value)
}

void	Setrotate_theta(int fd, int no, int n)
{
    gCmdList[no]. cmd = 'd';
    gCmdList[no]. para[0] = (short)n;
    gCmdList[no]. para[1] = (short)0;
    if (0 == no) gCmdList[no]. para[2] = (short)200;
    else         gCmdList[no]. para[2] = (short)0; // default
}

void	Setrotate_and_go(int fd, int no, int theta, int dist)
{
    if (30 < abs(theta)) {
        Setrotate_theta(fd, no, theta);
	return;
    }
    gCmdList[no]. cmd = 'j';
    gCmdList[no]. para[0] = (short)theta;
    gCmdList[no]. para[1] = (short)dist;
    if (0 == no) gCmdList[no]. para[2] = (short)300;
    else         gCmdList[no]. para[2] = (short)NORMALSPEED; // originally 0
}

void	Setarc(int fd, int no, int dir, int radius, int leng)
{
    gCmdList[no]. cmd = 'k';
    gCmdList[no]. para[0] = (short)dir;
    gCmdList[no]. para[1] = (short)radius;
    gCmdList[no]. para[2] = (short)leng;
}

void	Setnop(int fd, int no)
{
    gCmdList[no]. cmd = 'n';
    gCmdList[no]. para[0] = 0;
    gCmdList[no]. para[1] = 0;
    gCmdList[no]. para[2] = 0;
}

void	SendCmdList()
{
    f5call(fd, gCmdList);
}

void	Print2TeamLog(Obj *bptr, Obj *mptr, Obj *eptr, CmdList *mlist, CmdList *elist, int time)
{
    PrintObjPosition(bptr);
    PrintTeamObjPosition(mptr);
    PrintTeamObjPosition(eptr);
    PrintTeamCmdList(mlist);
    PrintTeamCmdList(elist);
    printf("%d\n", time);
}

void	PrintTeamLog(Obj *bptr, Obj *mptr, Obj *eptr, CmdList *list, int time)
{
    PrintObjPosition(bptr);
    PrintTeamObjPosition(mptr);
    PrintTeamObjPosition(eptr);
    PrintTeamCmdList(list);
    printf("%d\n", time);
}

void	PrintMotionLog(Obj *bptr, Obj *mptr, Obj *eptr, int time)
{
    PrintObjPosition(bptr);
    PrintTeamObjPosition(mptr);
    PrintTeamObjPosition(eptr);
    printf("%d\n", time);
}

void	PrintTeamObjPosition(Obj *ptr)
{
    int		i;

    for (i = 0; i < MAXPLAYER; i++) PrintObjPosition(&ptr[i]);
}

void	PrintObjPosition(Obj *ptr)
{
    printf("%d %d %d ", (int)(ptr -> x), (int)(ptr -> y), (int)(ptr -> dir));
}

void    PrintTeamCmdList(CmdList *ptr)
{
    int		i;

    for (i = 0; i < MAXPLAYER; i++) PrintCmdList(&ptr[i]);
}

void	PrintCmdList(CmdList *ptr)
{
    printf("%c %d %d %d ", ptr -> cmd, ptr -> para[0], ptr -> para[1], ptr -> para[2]);
}

void    PrintTeamReadableCmdList(CmdList *ptr)
{
    int		i;

    for (i = 0; i < MAXPLAYER; i++) PrintReadableCmdList(&ptr[i]);
}

void	PrintReadableCmdList(CmdList *ptr)
{
    switch (ptr -> cmd) {
    case 'g':
	printf("(go %d)  ", ptr -> para[0]);
	break;
    case 'd':
	printf("(rot %d)  ", ptr -> para[0]);
	break;
    case 'j':
	printf("(r&g %d %d)  ", ptr -> para[0], ptr -> para[1]);
	break;
    default:
	break;
    }
}

int	Token(char str[], FLOAT num[])
{
    char        *ptr, *ptr2, str2[BUFSIZ];
    int         j, n;

    n = 0;
    ptr = str;

    while (TRUE) {
        while ((SPC == *ptr) || (TAB == *ptr)) ptr++;
        if (((char)0 == *ptr) || (RET == *ptr) || (CMT == *ptr))
            return(n);
        ptr2 = ptr;
        j = 0;
        while ((SPC != *ptr) && (TAB != *ptr) && (CMT != *ptr) &&
               ((char)0 != *ptr) && (RET != *ptr)) {
            ptr++;
            j++;
        }
        strncpy(str2, ptr2, j);
        str2[j] = (char)0;
        if (NO == TestDigit(str2)) return(n);
        num[n] = atof(str2);
        n++;
    }
}

int	TestDigit(char str[])
{
    int         i = 0;

    while ((char)NULL != str[i]) {
        switch (str[i]) {
	case '0': case '1':
	case '2': case '3':
	case '4': case '5':
	case '6': case '7':
	case '8': case '9':
	case '.': case '+':
	case '-':
                break;
            default :
                return(NO);
	    }
        i++;
    }
    return(YES);
}
